@@ -24,6 +24,7 @@
 #include <dirent.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <stdio.h>
 #include <stdint.h>
 #include <stdlib.h>
 #ifdef HAVE_STATFS
@@ -36,6 +37,9 @@
       #define CHECK_DISK_SPACE 1
     #else
       #ifdef HAVE_SYS_MOUNT_H
+        #if defined(__OpenBSD__)
+          #include <sys/param.h>
+        #endif
         #include <sys/mount.h>
         #define CHECK_DISK_SPACE 1
       #endif
@@ -70,7 +74,6 @@
 #include "DaemonState.h"
 #include "OnDiskHandler.h"
 #include "PinotSettings.h"
-#include "PinotUtils.h"
 #include "ServerThreads.h"
 
 using namespace std;
@@ -318,6 +321,7 @@ DaemonState::DaemonState() :
 	m_isReindex(false),
 	m_reload(false),
 	m_flush(false),
+	m_crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName()),
 	m_pDiskMonitor(MonitorFactory::getMonitor()),
 	m_pDiskHandler(NULL),
 	m_crawlers(0)
@@ -487,7 +491,9 @@ void DaemonState::flush_and_reclaim(void)
 	IndexInterface *pIndex = PinotSettings::getInstance().getIndex(PinotSettings::getInstance().m_daemonIndexLocation);
 	if (pIndex != NULL)
 	{
+#ifdef HAVE_DBUS
 		DBusServletThread::flushIndexAndSignal(pIndex);
+#endif
 
 		delete pIndex;
 	}
@@ -522,12 +528,10 @@ void DaemonState::start(bool isReindex)
 	cout << "DaemonState::start: " << m_crawlQueue.size() << " locations to crawl" << endl;
 #endif
 
-	CrawlHistory crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName());
-
 	// Update all items status so that we can get rid of files from deleted sources
-	crawlHistory.updateItemsStatus(CrawlHistory::CRAWLING, CrawlHistory::TO_CRAWL, 0, true);
-	crawlHistory.updateItemsStatus(CrawlHistory::CRAWLED, CrawlHistory::TO_CRAWL, 0, true);
-	crawlHistory.updateItemsStatus(CrawlHistory::CRAWL_ERROR, CrawlHistory::TO_CRAWL, 0, true);
+	m_crawlHistory.updateItemsStatus(CrawlHistory::CRAWLING, CrawlHistory::TO_CRAWL, 0, true);
+	m_crawlHistory.updateItemsStatus(CrawlHistory::CRAWLED, CrawlHistory::TO_CRAWL, 0, true);
+	m_crawlHistory.updateItemsStatus(CrawlHistory::CRAWL_ERROR, CrawlHistory::TO_CRAWL, 0, true);
 
 	// Initiate crawling
 	start_crawling();
@@ -562,12 +566,11 @@ bool DaemonState::start_crawling(void)
 			}
 			else
 			{
-				CrawlHistory crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName());
 				set<string> deletedFiles;
 
 				// All files left with status TO_CRAWL belong to deleted sources
 				if ((m_pDiskHandler != NULL) &&
-					(crawlHistory.getItems(CrawlHistory::TO_CRAWL, deletedFiles) > 0))
+					(m_crawlHistory.getItems(CrawlHistory::TO_CRAWL, deletedFiles) > 0))
 				{
 #ifdef DEBUG
 					cout << "DaemonState::start_crawling: " << deletedFiles.size() << " orphaned files" << endl;
@@ -579,7 +582,7 @@ bool DaemonState::start_crawling(void)
 						m_pDiskHandler->fileDeleted(fileIter->substr(7));
 
 						// Delete this item
-						crawlHistory.deleteItem(*fileIter);
+						m_crawlHistory.deleteItem(*fileIter);
 					}
 				}
 			}
@@ -663,10 +666,8 @@ void DaemonState::on_thread_end(WorkerThread *pThread)
 		if ((errorNum > 0) &&
 			(indexedUrl.empty() == false))
 		{
-			CrawlHistory crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName());
-
 			// An entry should already exist for this
-			crawlHistory.updateItem(indexedUrl, CrawlHistory::CRAWL_ERROR, time(NULL), errorNum);
+			m_crawlHistory.updateItem(indexedUrl, CrawlHistory::CRAWL_ERROR, time(NULL), errorNum);
 		}
 	}
 	else if (type == "UnindexingThread")
