@@ -27,6 +27,10 @@
 #include <signal.h>
 #include <time.h>
 #include <errno.h>
+#ifdef __OpenBSD__
+#include <sys/param.h>
+#include <sys/sysctl.h>
+#endif
 #include <exception>
 #include <iostream>
 #include <fstream>
@@ -45,8 +49,6 @@
 #include "Url.h"
 #include "HtmlFilter.h"
 #include "FilterUtils.h"
-#include "ActionQueue.h"
-#include "CrawlHistory.h"
 #include "DownloaderFactory.h"
 #include "FilterWrapper.h"
 #include "ModuleFactory.h"
@@ -274,6 +276,7 @@ unsigned int ThreadsManager::m_nextThreadId = 1;
 ThreadsManager::ThreadsManager(const string &defaultIndexLocation,
 	unsigned int maxIndexThreads, unsigned int maxThreadsTime,
 	bool scanLocalFiles) :
+	m_actionQueue(PinotSettings::getInstance().getHistoryDatabaseName(), get_application_name()),
 	m_defaultIndexLocation(defaultIndexLocation),
 	m_maxIndexThreads(maxIndexThreads),
 	m_backgroundThreadsCount(0),
@@ -285,9 +288,21 @@ ThreadsManager::ThreadsManager(const string &defaultIndexLocation,
 	pthread_rwlock_init(&m_threadsLock, NULL);
 	pthread_rwlock_init(&m_listsLock, NULL);
 
+#ifdef __OpenBSD__
+	int mib[2], ncpus;
+
+	mib[0] = CTL_HW;
+	mib[1] = HW_NCPU;
+	size_t len = sizeof(ncpus);
+	if (sysctl(mib, 2, &ncpus, &len, NULL, 0) > 0)
+	{
+		m_numCPUs = ncpus;
+	}
+#else
 #ifdef HAVE_SYSCONF
 	m_numCPUs = sysconf(_SC_NPROCESSORS_ONLN);
 #endif
+#endif
 }
 
 ThreadsManager::~ThreadsManager()
@@ -492,9 +507,7 @@ void ThreadsManager::clear_queues(void)
 
 		unlock_lists();
 
-		ActionQueue actionQueue(PinotSettings::getInstance().getHistoryDatabaseName(), get_application_name());
-
-		actionQueue.expireItems(time(NULL));
+		m_actionQueue.expireItems(time(NULL));
 	}
 }
 
@@ -683,9 +696,7 @@ ustring ThreadsManager::queue_index(const DocumentInfo &docInfo)
 
 	if (addToQueue == true)
 	{
-		ActionQueue actionQueue(PinotSettings::getInstance().getHistoryDatabaseName(), get_application_name());
-
-		actionQueue.pushItem(ActionQueue::INDEX, docInfo);
+		m_actionQueue.pushItem(ActionQueue::INDEX, docInfo);
 
 		return "";
 	}
@@ -726,7 +737,6 @@ bool ThreadsManager::pop_queue(const string &urlWasIndexed)
 		// Get an item ?
 		if (getItem == true)
 		{
-			ActionQueue actionQueue(PinotSettings::getInstance().getHistoryDatabaseName(), get_application_name());
 			ActionQueue::ActionType type;
 			DocumentInfo docInfo;
 			string previousLocation;
@@ -734,7 +744,7 @@ bool ThreadsManager::pop_queue(const string &urlWasIndexed)
 			// Assume the queue is empty
 			emptyQueue = true;
 
-			while (actionQueue.popItem(type, docInfo) == true)
+			while (m_actionQueue.popItem(type, docInfo) == true)
 			{
 				ustring status;
 
@@ -1714,6 +1724,7 @@ MonitorThread::MonitorThread(MonitorInterface *pMonitor, MonitorHandler *pHandle
 	WorkerThread(),
 	m_ctrlReadPipe(-1),
 	m_ctrlWritePipe(-1),
+	m_crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName()),
 	m_pMonitor(pMonitor),
 	m_pHandler(pHandler),
 	m_checkHistory(checkHistory)
@@ -1758,7 +1769,6 @@ void MonitorThread::stop(void)
 
 void MonitorThread::processEvents(void)
 {
-	CrawlHistory crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName());
 	queue<MonitorEvent> events;
 
 #ifdef DEBUG
@@ -1834,7 +1844,7 @@ void MonitorThread::processEvents(void)
 				{
 					m_pHandler->fileModified(event.m_location);
 				}
-				else if (crawlHistory.hasItem("file://" + event.m_location, status, itemDate) == true)
+				else if (m_crawlHistory.hasItem("file://" + event.m_location, status, itemDate) == true)
 				{
 					// Was the file actually modified ?
 					if ((stat(event.m_location.c_str(), &fileStat) == 0) &&
@@ -1999,7 +2009,7 @@ sigc::signal2<void, DocumentInfo, bool>& DirectoryScannerThread::getFileFoundSig
 	return m_signalFileFound;
 }
 
-void DirectoryScannerThread::cacheUpdate(const string &location, time_t itemDate)
+void DirectoryScannerThread::recordCrawled(const string &location, time_t itemDate)
 {
 	// Nothing to do by default
 }
@@ -2362,7 +2372,7 @@ bool DirectoryScannerThread::scanEntry(const string &entryName,
 	else if ((itemExists == false) ||
 		(reportFile == true))
 	{
-		cacheUpdate(location, fileStat.st_mtime);
+		recordCrawled(location, fileStat.st_mtime);
 	}
 
 	// If a major error occured, this won't be true
