@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -22,10 +22,12 @@
 #include <string>
 #include <set>
 
+#include "Visibility.h"
+
 using namespace std;
 
 /// This represents a query.
-class QueryProperties
+class PINOT_EXPORT QueryProperties
 {
 	public:
 		typedef enum { XAPIAN_QP = 0, XESAM_QL, XESAM_UL } QueryType;
@@ -62,6 +64,11 @@ class QueryProperties
 		/// Gets the language to use for stemming.
 		string getStemmingLanguage(void) const;
 
+		/// Sets whether the query is sensitive to diacritics.
+		void setDiacriticSensitive(bool sensitive);
+		/// Gets whether the query is sensitive to diacritics.
+		bool getDiacriticSensitive(void) const;
+
 		/// Sets the query string.
 		void setFreeQuery(const string &freeQuery);
 		/// Gets the query string.
@@ -98,6 +105,7 @@ class QueryProperties
 		QueryType m_type;
 		SortOrder m_order;
 		string m_language;
+		bool m_diacriticSensitive;
 		string m_freeQuery;
 		string m_freeQueryWithoutFilters;
 		unsigned int m_resultsCount;
