@@ -31,13 +31,19 @@
 
 using std::string;
 using std::set;
-using std::cout;
+using std::vector;
+using std::clog;
 using std::endl;
 
 ActionQueue::ActionQueue(const string &database, const string queueId) :
-	SQLiteBase(database),
+	SQLiteBase(database, false, false),
 	m_queueId(queueId)
 {
+	prepareStatement("select-url",
+		"SELECT Url FROM ActionQueue WHERE QueueId=? AND Url=?;");
+	prepareStatement("select-oldest-url",
+		"SELECT Type, Info FROM ActionQueue "
+		"WHERE QueueId=? ORDER BY Date DESC LIMIT 1;");
 }
 
 ActionQueue::~ActionQueue()
@@ -92,7 +98,7 @@ bool ActionQueue::create(const string &database)
 	if (db.executeSimpleStatement("SELECT * FROM ActionQueue LIMIT 1;") == false)
 	{
 #ifdef DEBUG
-		cout << "ActionQueue::create: ActionQueue doesn't exist" << endl;
+		clog << "ActionQueue::create: ActionQueue doesn't exist" << endl;
 #endif
 		// Create the table
 		if (db.executeSimpleStatement("CREATE TABLE ActionQueue (QueueId VARCHAR(255), \
@@ -109,21 +115,22 @@ bool ActionQueue::create(const string &database)
 /// Pushes an item.
 bool ActionQueue::pushItem(ActionType type, const DocumentInfo &docInfo)
 {
+	vector<string> values;
 	string url(docInfo.getLocation());
 	string info(docInfo.serialize());
 	bool update = false, success = false;
 
 	// Is there already an item for this URL ?
-	SQLResults *results = executeStatement("SELECT Url FROM ActionQueue \
-		WHERE QueueId='%q' AND Url='%q';",
-		m_queueId.c_str(), Url::escapeUrl(url).c_str());
+	values.push_back(m_queueId);
+	values.push_back(Url::escapeUrl(url));
+	SQLResults *results = executePreparedStatement("select-url", values);
 	if (results != NULL)
 	{
 		SQLRow *row = results->nextRow();
 		if (row != NULL)
 		{
 #ifdef DEBUG
-			cout << "ActionQueue::pushItem: item "
+			clog << "ActionQueue::pushItem: item "
 				<< Url::unescapeUrl(row->getColumn(0)) << " exists" << endl;
 #endif
 			update = true;
@@ -152,7 +159,7 @@ bool ActionQueue::pushItem(ActionType type, const DocumentInfo &docInfo)
 	if (results != NULL)
 	{
 #ifdef DEBUG
-		cout << "ActionQueue::pushItem: queue " << m_queueId
+		clog << "ActionQueue::pushItem: queue " << m_queueId
 			<< ": " << type << " on " << url << ", " << update << endl;
 #endif
 		success = true;
@@ -175,7 +182,7 @@ bool ActionQueue::popItem(ActionType &type, DocumentInfo &docInfo)
 	}
 	url = docInfo.getLocation();
 #ifdef DEBUG
-	cout << "ActionQueue::popItem: queue " << m_queueId
+	clog << "ActionQueue::popItem: queue " << m_queueId
 		<< ": " << type << " on " << url << endl;
 #endif
 
@@ -194,11 +201,11 @@ bool ActionQueue::popItem(ActionType &type, DocumentInfo &docInfo)
 
 bool ActionQueue::getOldestItem(ActionType &type, DocumentInfo &docInfo)
 {
+	vector<string> values;
 	bool success = false;
 
-	SQLResults *results = executeStatement("SELECT Type, Info FROM ActionQueue \
-		WHERE QueueId='%q' ORDER BY Date DESC LIMIT 1;",
-		m_queueId.c_str());
+	values.push_back(m_queueId);
+	SQLResults *results = executePreparedStatement("select-oldest-url", values);
 	if (results != NULL)
 	{
 		SQLRow *row = results->nextRow();
@@ -248,6 +255,11 @@ bool ActionQueue::expireItems(time_t expiryDate)
 {
 	bool success = false;
 
+	if (beginTransaction() == false)
+	{
+		return false;
+	}
+
 	SQLResults *results = executeStatement("DELETE FROM ActionQueue \
 		WHERE QueueId='%q' AND Date<'%d';",
 		m_queueId.c_str(), expiryDate);
@@ -257,5 +269,11 @@ bool ActionQueue::expireItems(time_t expiryDate)
 		delete results;
 	}
 
+	if (endTransaction() == false)
+	{
+		return false;
+	}
+
 	return success;
 }
+
