@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2008 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,15 +21,17 @@
 #include <stdarg.h>
 #include <unistd.h>
 #include <algorithm>
+#include <utility>
 #include <iostream>
 
 #include "SQLiteBase.h"
 
-using std::cout;
-using std::cerr;
+using std::clog;
 using std::endl;
 using std::string;
 using std::vector;
+using std::map;
+using std::pair;
 
 static int busyHandler(void *pData, int lockNum)
 {
@@ -37,8 +39,22 @@ static int busyHandler(void *pData, int lockNum)
 	return 1;
 }
 
+// A function object to delete statements with for_each()
+struct DeleteStatementsFunc
+{
+	public:
+		void operator()(map<string, sqlite3_stmt*>::value_type &p)
+		{
+			if (p.second != NULL)
+			{
+				sqlite3_finalize(p.second);
+			}
+		}
+};
+
 SQLiteRow::SQLiteRow(const vector<string> &rowColumns, unsigned int nColumns) :
-	SQLRow(nColumns)
+	SQLRow(nColumns),
+	m_pStatement(NULL)
 {
 	if (rowColumns.empty() == false)
 	{
@@ -55,12 +71,32 @@ SQLiteRow::SQLiteRow(const vector<string> &rowColumns, unsigned int nColumns) :
 	}
 }
 
+SQLiteRow::SQLiteRow(sqlite3_stmt *pStatement, unsigned int nColumns) :
+	SQLRow(nColumns),
+	m_pStatement(pStatement)
+{
+}
+
 SQLiteRow::~SQLiteRow()
 {
 }
 
 string SQLiteRow::getColumn(unsigned int nColumn) const
 {
+	if (m_pStatement != NULL)
+	{
+		const unsigned char *pTextColumn = sqlite3_column_text(m_pStatement, nColumn);
+		if (pTextColumn != NULL)
+		{
+			return (const char*)pTextColumn;
+		}
+#ifdef DEBUG
+		clog << "SQLiteRow::getColumn: couldn't get column " << nColumn << endl;
+#endif
+
+		return "";
+	}
+
 	if (nColumn < m_nColumns)
 	{
 		vector<string>::const_iterator colIter = m_columns.begin();
@@ -81,7 +117,8 @@ string SQLiteRow::getColumn(unsigned int nColumn) const
 
 SQLiteResults::SQLiteResults(char **results, unsigned long nRows, unsigned int nColumns) :
 	SQLResults(nRows, nColumns),
-	m_results(results)
+	m_results(results),
+	m_pStatement(NULL)
 {
 	// Check we actually have results
 	if ((m_results == NULL) ||
@@ -92,13 +129,43 @@ SQLiteResults::SQLiteResults(char **results, unsigned long nRows, unsigned int n
 	}
 }
 
+SQLiteResults::SQLiteResults(sqlite3_stmt *pStatement) :
+	SQLResults(0, sqlite3_column_count(pStatement)),
+	m_results(NULL),
+	m_pStatement(pStatement)
+{
+}
+
 SQLiteResults::~SQLiteResults()
 {
-	sqlite3_free_table(m_results);
+	if (m_results != NULL)
+	{
+		sqlite3_free_table(m_results);
+	}
+	if (m_pStatement != NULL)
+	{
+		rewind();
+	}
+}
+
+bool SQLiteResults::hasMoreRows(void) const
+{
+	if (m_pStatement != NULL)
+	{
+		// There's no way to tell
+		return true;
+	}
+
+	return SQLResults::hasMoreRows();
 }
 
 string SQLiteResults::getColumnName(unsigned int nColumn) const
 {
+	if (m_pStatement != NULL)
+	{
+		return sqlite3_column_name(m_pStatement, (int)nColumn);
+	}
+
 	if (nColumn < m_nColumns)
 	{
 		return m_results[nColumn];
@@ -109,6 +176,20 @@ string SQLiteResults::getColumnName(unsigned int nColumn) const
 
 SQLRow *SQLiteResults::nextRow(void)
 {
+	if (m_pStatement != NULL)
+	{
+		if (sqlite3_step(m_pStatement) == SQLITE_ROW)
+		{
+			++m_nCurrentRow;
+			return new SQLiteRow(m_pStatement, m_nColumns);
+		}
+#ifdef DEBUG
+		clog << "SQLiteResults::nextRow: no more row" << endl;
+#endif
+
+		return NULL;
+	}
+
 	if ((m_nCurrentRow < 0) ||
 		(m_nCurrentRow >= m_nRows))
 	{
@@ -136,8 +217,20 @@ SQLRow *SQLiteResults::nextRow(void)
 	return new SQLiteRow(rowColumns, m_nColumns);
 }
 
-SQLiteBase::SQLiteBase(const string &databaseName, bool onDemand) :
-	SQLDB(databaseName),
+bool SQLiteResults::rewind(void)
+{
+	SQLResults::rewind();
+	if (m_pStatement != NULL)
+	{
+		sqlite3_reset(m_pStatement);
+	}
+
+	return true;
+}
+
+SQLiteBase::SQLiteBase(const string &databaseName,
+	bool readOnly, bool onDemand) :
+	SQLDB(databaseName, readOnly),
 	m_onDemand(onDemand),
 	m_pDatabase(NULL)
 {
@@ -164,7 +257,7 @@ bool SQLiteBase::check(const string &databaseName)
 		(!S_ISREG(dbStat.st_mode)))
 	{
 		// It exists, but it's not a file as expected
-		cerr << databaseName << " is not a file" << endl;
+		clog << databaseName << " is not a file" << endl;
 		return false;
 	}
 
@@ -173,20 +266,28 @@ bool SQLiteBase::check(const string &databaseName)
 
 void SQLiteBase::open(void)
 {
+	int openFlags = SQLITE_OPEN_READWRITE|SQLITE_OPEN_CREATE;
+
+	if (m_readOnly == true)
+	{
+		openFlags = SQLITE_OPEN_READONLY;
+	}
+
 	// Open the new database
-	if (sqlite3_open(m_databaseName.c_str(), &m_pDatabase) != SQLITE_OK)
+	if (sqlite3_open_v2(m_databaseName.c_str(), &m_pDatabase,
+		openFlags, NULL) != SQLITE_OK)
 	{
 		// An handle is returned even when an error occurs !
 		if (m_pDatabase != NULL)
 		{
-			cerr << sqlite3_errmsg(m_pDatabase) << endl;
+			clog << sqlite3_errmsg(m_pDatabase) << endl;
 			close();
 			m_pDatabase = NULL;
 		}
 	}
 	else if (m_pDatabase == NULL)
 	{
-		cerr << "Couldn't open " << m_databaseName << endl;
+		clog << "Couldn't open " << m_databaseName << endl;
 	}
 	else
 	{
@@ -199,6 +300,9 @@ void SQLiteBase::close(void)
 {
 	if (m_pDatabase != NULL)
 	{
+		for_each(m_statements.begin(), m_statements.end(),
+			DeleteStatementsFunc());
+
 		sqlite3_close(m_pDatabase);
 	}
 }
@@ -249,12 +353,34 @@ bool SQLiteBase::alterTable(const string &tableName,
 	sql += tableName;
 	sql += "_backup; COMMIT;";
 #ifdef DEBUG
-	cout << "SQLiteBase::alterTable: " << sql << endl;
+	clog << "SQLiteBase::alterTable: " << sql << endl;
 #endif
 
 	return executeSimpleStatement(sql);
 }
 
+bool SQLiteBase::beginTransaction(void)
+{
+	if (m_onDemand == true)
+	{
+		// Not applicable
+		return true;
+	}
+
+	return executeSimpleStatement("BEGIN TRANSACTION;");
+}
+
+bool SQLiteBase::endTransaction(void)
+{
+	if (m_onDemand == true)
+	{
+		// Not applicable
+		return true;
+	}
+
+	return executeSimpleStatement("END TRANSACTION;");
+}
+
 bool SQLiteBase::executeSimpleStatement(const string &sql)
 {
 	char *errMsg = NULL;
@@ -281,7 +407,7 @@ bool SQLiteBase::executeSimpleStatement(const string &sql)
 	{
 		if (errMsg != NULL)
 		{
-			cerr << "Statement <" << sql << "> failed: " << errMsg << endl;
+			clog << m_databaseName << ": statement <" << sql << "> failed: " << errMsg << endl;
 
 			sqlite3_free(errMsg);
 		}
@@ -324,7 +450,7 @@ SQLResults *SQLiteBase::executeStatement(const char *sqlFormat, ...)
 	if (numChars <= 0)
 	{
 #ifdef DEBUG
-		cout << "SQLiteBase::executeStatement: couldn't format statement" << endl;
+		clog << "SQLiteBase::executeStatement: couldn't format statement" << endl;
 #endif
 		if (m_onDemand == true)
 		{
@@ -336,7 +462,7 @@ SQLResults *SQLiteBase::executeStatement(const char *sqlFormat, ...)
 	{
 		// Not enough space
 #ifdef DEBUG
-		cout << "SQLiteBase::executeStatement: not enough space (" << numChars << ")" << endl;
+		clog << "SQLiteBase::executeStatement: not enough space (" << numChars << ")" << endl;
 #endif
 		if (m_onDemand == true)
 		{
@@ -350,7 +476,7 @@ SQLResults *SQLiteBase::executeStatement(const char *sqlFormat, ...)
 	if (stringBuff == NULL)
 	{
 #ifdef DEBUG
-		cout << "SQLiteBase::executeStatement: couldn't format statement" << endl;
+		clog << "SQLiteBase::executeStatement: couldn't format statement" << endl;
 #endif
 		if (m_onDemand == true)
 		{
@@ -364,15 +490,15 @@ SQLResults *SQLiteBase::executeStatement(const char *sqlFormat, ...)
 	char *errMsg;
 	int nRows, nColumns;
 	if (sqlite3_get_table(m_pDatabase,
-			stringBuff,
-			&results,
-			&nRows,
-			&nColumns,
-			&errMsg) != SQLITE_OK)
+		stringBuff,
+		&results,
+		&nRows,
+		&nColumns,
+		&errMsg) != SQLITE_OK)
 	{
 		if (errMsg != NULL)
 		{
-			cerr << "Statement <" << stringBuff << "> failed: " << errMsg << endl;
+			clog << m_databaseName << ": statement <" << stringBuff << "> failed: " << errMsg << endl;
 
 			sqlite3_free(errMsg);
 		}
@@ -392,3 +518,84 @@ SQLResults *SQLiteBase::executeStatement(const char *sqlFormat, ...)
 
 	return pResults;
 }
+
+bool SQLiteBase::prepareStatement(const string &statementId,
+	const string &sqlFormat)
+{
+	if ((sqlFormat.empty() == true) ||
+		(m_onDemand == true) ||
+		(m_pDatabase == NULL))
+	{
+		return false;
+	}
+
+	map<string, sqlite3_stmt*>::iterator statIter = m_statements.find(statementId);
+	if (statIter != m_statements.end())
+	{
+#ifdef DEBUG
+		clog << "SQLiteBase::prepareStatement: invalid statement ID " << statementId << endl;
+#endif
+		return false;
+	}
+
+	sqlite3_stmt *pStatement = NULL;
+	if (sqlite3_prepare_v2(m_pDatabase,
+		sqlFormat.c_str(),
+		(int)sqlFormat.length(),
+		&pStatement,
+		NULL) == SQLITE_OK)
+	{
+		if (pStatement != NULL)
+		{
+#ifdef DEBUG
+			clog << "SQLiteBase::prepareStatement: compiled statement ID " << statementId << endl;
+#endif
+			m_statements.insert(pair<string, sqlite3_stmt*>(statementId, pStatement));
+			return true;
+		}
+	}
+
+	clog << m_databaseName << ": failed to compile statement <" << sqlFormat << ">" << endl;
+
+	return false;
+}
+
+SQLResults * SQLiteBase::executePreparedStatement(const string &statementId,
+	const vector<string> &values)
+{
+	int paramIndex = 1;
+
+	map<string, sqlite3_stmt*>::iterator statIter = m_statements.find(statementId);
+	if (statIter == m_statements.end())
+	{
+#ifdef DEBUG
+		clog << "SQLiteBase::executePreparedStatement: invalid statement ID " << statementId << endl;
+#endif
+		return NULL;
+	}
+
+	// Bind values
+	// The left-most parameter's index is 1
+	for (vector<string>::const_iterator valueIter = values.begin();
+		valueIter != values.end(); ++valueIter, ++paramIndex)
+	{
+		int errorCode = sqlite3_bind_text(statIter->second, paramIndex,
+			valueIter->c_str(), -1, SQLITE_TRANSIENT);
+		if (errorCode != SQLITE_OK)
+		{
+			clog << m_databaseName << ": failed to bind parameter " << paramIndex
+				<< " to statement ID " << statementId << ": error " << errorCode << endl;
+			return NULL;
+		}
+#ifdef DEBUG
+		clog << "SQLiteBase::executePreparedStatement: bound parameter " << paramIndex
+			<< " size " << valueIter->length() << " to statement ID " << statementId << endl;
+#endif
+	}
+
+#ifdef DEBUG
+	clog << "SQLiteBase::executePreparedStatement: statement ID " << statementId << endl;
+#endif
+	return new SQLiteResults(statIter->second);
+}
+
