@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2008 Fabrice Colin
+ *  Copyright 2007-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,11 +16,16 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <iostream>
 #include <set>
 
+#include "config.h"
+#include "Memory.h"
 #include "MIMEScanner.h"
 #include "StringManip.h"
 #include "TimeConverter.h"
@@ -30,6 +35,7 @@
 #include "FilterUtils.h"
 
 #define UNSUPPORTED_TYPE "X-Unsupported"
+#define SIZE_THRESHOLD 5242880
 
 using std::cout;
 using std::cerr;
@@ -41,6 +47,31 @@ using std::map;
 set<string> FilterUtils::m_types;
 map<string, string> FilterUtils::m_typeAliases;
 
+ReducedAction::ReducedAction()
+{
+}
+
+ReducedAction::~ReducedAction()
+{
+}
+
+bool ReducedAction::positionFilter(const Document &doc, Dijon::Filter *pFilter)
+{
+	return false;
+}
+
+bool ReducedAction::isReduced(const Document &doc)
+{
+	// Is it reduced to plain text ?
+	if ((doc.getType().length() >= 10) &&
+		(doc.getType().substr(0, 10) == "text/plain"))
+	{
+		return true;
+	}
+
+	return false;
+}
+
 FilterUtils::FilterUtils()
 {
 }
@@ -125,34 +156,35 @@ bool FilterUtils::isSupportedType(const string &mimeType)
 		return true;
 	}
 
-	if (Dijon::FilterFactory::isSupportedType(mimeType) == false)
+	if (Dijon::FilterFactory::isSupportedType(mimeType) == true)
 	{
-		set<string> parentTypes;
+		return true;
+	}
 
-		if (m_types.empty() == true)
-		{
-			Dijon::FilterFactory::getSupportedTypes(m_types);
-		}
+	if (m_types.empty() == true)
+	{
+		Dijon::FilterFactory::getSupportedTypes(m_types);
+	}
 
-		// Try that type's parents
-		MIMEScanner::getParentTypes(mimeType, m_types, parentTypes);
-		for (set<string>::const_iterator parentIter = parentTypes.begin();
-			parentIter != parentTypes.end(); ++parentIter)
+	// Try that type's parents
+	set<string> parentTypes;
+	MIMEScanner::getParentTypes(mimeType, m_types, parentTypes);
+	for (set<string>::const_iterator parentIter = parentTypes.begin();
+		parentIter != parentTypes.end(); ++parentIter)
+	{
+		if (Dijon::FilterFactory::isSupportedType(*parentIter) == true)
 		{
-			if (Dijon::FilterFactory::isSupportedType(*parentIter) == true)
-			{
-				// Add an alias
-				m_typeAliases[mimeType] = *parentIter;
-				return true;
-			}
+			// Add an alias
+			m_typeAliases[mimeType] = *parentIter;
+			return true;
 		}
+	}
 #ifdef DEBUG
-		cout << "FilterUtils::isSupportedType: no valid parent for " << mimeType << endl;
+	cout << "FilterUtils::isSupportedType: no valid parent for " << mimeType << endl;
 #endif
 
-		// This type has no valid parent
-		m_typeAliases[mimeType] = UNSUPPORTED_TYPE;
-	}
+	// This type has no valid parent
+	m_typeAliases[mimeType] = UNSUPPORTED_TYPE;
 
 	return false;
 }
@@ -172,7 +204,7 @@ bool FilterUtils::feedFilter(const Document &doc, Dijon::Filter *pFilter)
 	}
 
 	if ((urlObj.getProtocol() == "file") &&
-		(location .length() > 7))
+		(location.length() > 7))
 	{
 		fileName = location.substr(7);
 	}
@@ -183,9 +215,47 @@ bool FilterUtils::feedFilter(const Document &doc, Dijon::Filter *pFilter)
 	{
 		fedInput = pFilter->set_document_data(pData, dataLength);
 	}
+	// ... to feeding the data through a temporary file
+	if ((fedInput == false) &&
+		((dataLength > 0) && (pData != NULL)) &&
+		(pFilter->is_data_input_ok(Dijon::Filter::DOCUMENT_FILE_NAME) == true))
+	{
+		char inTemplate[18] = "/tmp/filterXXXXXX";
+
+#ifdef HAVE_MKSTEMP
+		int inFd = mkstemp(inTemplate);
+#else
+		int inFd = -1;
+		char *pInFile = mktemp(inTemplate);
+		if (pInFile != NULL)
+		{
+			inFd = open(pInFile, O_RDONLY);
+		}
+#endif
+		if (inFd != -1)
+		{
+#ifdef DEBUG
+			cout << "FilterUtils::feedFilter: feeding temporary file " << inTemplate << endl;
+#endif
+
+			// Save the data
+			if (write(inFd, (const void*)pData, dataLength) != -1)
+			{
+				fedInput = pFilter->set_document_file(inTemplate, true);
+				if (fedInput == false)
+				{
+					// We might as well delete the file now
+					unlink(inTemplate);
+				}
+			}
+
+			close(inFd);
+		}
+	}
 	// ... to feeding the file
 	if ((fedInput == false) &&
-		(fileName.empty() == false))
+		(fileName.empty() == false) &&
+		(doc.getInternalPath().empty() == true))
 	{ 
 		if (pFilter->is_data_input_ok(Dijon::Filter::DOCUMENT_FILE_NAME) == true)
 		{
@@ -216,45 +286,13 @@ bool FilterUtils::feedFilter(const Document &doc, Dijon::Filter *pFilter)
 			{
 				fedInput = pFilter->set_document_data(pData, dataLength);
 			}
-			else
-			{
-				// The file may be empty
-				fedInput = pFilter->set_document_data(" ", 1);
-			}
-		}
-	}
-	// ... to feeding data through a temporary file
-	if ((fedInput == false) &&
-		((dataLength > 0) && (pData != NULL)) &&
-		(pFilter->is_data_input_ok(Dijon::Filter::DOCUMENT_FILE_NAME) == true))
-	{
-		char inTemplate[18] = "/tmp/filterXXXXXX";
-
-		int inFd = mkstemp(inTemplate);
-		if (inFd != -1)
-		{
-#ifdef DEBUG
-			cout << "FilterUtils::feedFilter: feeding temporary file " << inTemplate << endl;
-#endif
-
-			// Save the data
-			if (write(inFd, (const void*)pData, dataLength) != -1)
-			{
-				fedInput = pFilter->set_document_file(inTemplate, true);
-				if (fedInput == false)
-				{
-					// We might as well delete the file now
-					unlink(inTemplate);
-				}
-			}
-
-			close(inFd);
+			// Else, the file may be empty
 		}
 	}
 
 	if (fedInput == false)
 	{
-		cerr << "Couldn't feed filter for " << doc.getLocation() << endl;
+		cerr << "Couldn't feed filter for " << doc.getLocation(true) << endl;
 
 		return false;
 	}
@@ -265,7 +303,8 @@ bool FilterUtils::feedFilter(const Document &doc, Dijon::Filter *pFilter)
 bool FilterUtils::populateDocument(Document &doc, Dijon::Filter *pFilter)
 {
 	string charset, uri, ipath;
-	bool checkType = false;
+	off_t size = 0;
+	bool checkDataType = false, checkFileType = false;
 
 	if (pFilter == NULL)
 	{
@@ -299,16 +338,20 @@ bool FilterUtils::populateDocument(Document &doc, Dijon::Filter *pFilter)
 
 			if (mimeType == "scan")
 			{
-				checkType = true;
+				checkDataType = true;
+			}
+			else if (mimeType == "scantitle")
+			{
+				checkFileType = true;
 			}
 			else
 			{
-				doc.setType(StringManip::toLowerCase(metaIter->second));
+				doc.setType(mimeType);
 			}
 		}
 		else if (metaIter->first == "size")
 		{
-			off_t size = (off_t)atoi(metaIter->second.c_str());
+			size = (off_t)atoi(metaIter->second.c_str());
 
 			if (size > 0)
 			{
@@ -323,7 +366,7 @@ bool FilterUtils::populateDocument(Document &doc, Dijon::Filter *pFilter)
 			uri = metaIter->second;
 
 			if ((uri.length() >= 18) &&
-				(uri.substr(0, 18) == "file:///tmp/filter"))
+				(uri.find(":///tmp/filter") != string::npos))
 			{
 				// We fed the filter a temporary file
 				uri.clear();
@@ -337,46 +380,229 @@ bool FilterUtils::populateDocument(Document &doc, Dijon::Filter *pFilter)
 	}
 	if (ipath.empty() == false)
 	{
-		string location(doc.getLocation());
+		string currentIPath(doc.getInternalPath());
 
-		location += "?";
-		location += ipath;
-		doc.setLocation(location);
-	}
-
-	TextConverter converter(20);
-
-	// Content and title may have to be converted
-	map<string, string>::const_iterator contentIter = metaData.find("content");
-	if (contentIter != metaData.end())
-	{
-		string utf8Data(converter.toUTF8(contentIter->second, charset));
-
-		if (converter.getErrorsCount() > 0)
+		if (currentIPath.empty() == false)
 		{
-			cerr << doc.getLocation() << " may not have been fully converted to UTF-8" << endl;
+			currentIPath += "&next&";
 		}
-		doc.setData(utf8Data.c_str(), utf8Data.length());
-
-		if (checkType == true)
-		{
-			doc.setType(MIMEScanner::scanData(utf8Data.c_str(), utf8Data.length()));
-		}
-
+		currentIPath += ipath;
+		doc.setInternalPath(currentIPath);
 #ifdef DEBUG
-		cout << "FilterUtils::populateDocument: set " << utf8Data.length() << "/" << contentIter->second.length()
-			<< " bytes, converted from charset " << charset << endl;
+		cout << "FilterUtils::populateDocument: ipath " << currentIPath << endl;
 #endif
 	}
-	contentIter = metaData.find("title");
-	if (contentIter != metaData.end())
+
+	// Content and title may have to be converted
+	TextConverter converter(20);
+
+	map<string, string>::const_iterator contentIter = metaData.find("title");
+	if ((contentIter != metaData.end()) &&
+		(contentIter->second.empty() == false))
 	{
-		doc.setTitle(converter.toUTF8(contentIter->second, charset));
+		dstring nonUTF8Title(contentIter->second.c_str(), contentIter->second.length());
+
+		dstring utf8Data(converter.toUTF8(nonUTF8Title, charset));
+
+		doc.setTitle(string(utf8Data.c_str(), utf8Data.length()));
+	}
+
+	const dstring &content = pFilter->get_content();
+	if (content.empty() == false)
+	{
+		// Scan for the MIME type ?
+		if (checkFileType == true)
+		{
+			// Assume the title is actually a file name
+			string mimeType(MIMEScanner::scanFile(doc.getTitle()));
+
+			if ((mimeType.empty() == true) ||
+				(mimeType == "application/octet-stream"))
+			{
+				// Revert to scanning the content 
+				checkDataType = true;
+			}
+			else
+			{
+				doc.setType(mimeType);
+			}
+		}
+		if (checkDataType == true)
+		{
+			doc.setType(MIMEScanner::scanData(content.c_str(), content.length()));
+		}
+
+		if (doc.getType().substr(0, 10) == "text/plain")
+		{
+			dstring utf8Data(converter.toUTF8(content, charset));
+
+			if (converter.getErrorsCount() > 0)
+			{
+				cerr << doc.getLocation(true) << " may not have been fully converted to UTF-8" << endl;
+			}
+			doc.setData(utf8Data.c_str(), utf8Data.length());
+		}
+		else
+		{
+			doc.setData(content.c_str(), content.length());
+		}
+	}
+
+	// If the document is big'ish, try and reclaim memory
+	int inUse = Memory::getUsage();
+	if ((size > SIZE_THRESHOLD) ||
+		(content.length() > SIZE_THRESHOLD))
+	{
+		Memory::reclaim();
 	}
 
 	return true;
 }
 
+bool FilterUtils::filterDocument(const Document &doc, const string &originalType,
+	ReducedAction &action)
+{
+	Dijon::Filter *pFilter = FilterUtils::getFilter(doc.getType());
+	bool fedFilter = false, positionedFilter = false, docSuccess = false, finalSuccess = false;
+
+	if (pFilter != NULL)
+	{
+		fedFilter = FilterUtils::feedFilter(doc, pFilter);
+	}
+	positionedFilter = action.positionFilter(doc, pFilter);
+
+	if (fedFilter == false)
+	{
+		Document docCopy(doc);
+
+		if (docCopy.getTitle().empty() == true)
+		{
+			Url urlObj(doc.getLocation());
+
+			// Default to the file name as title
+			docCopy.setTitle(urlObj.getFile());
+		}
+
+		// Take the appropriate action now
+		finalSuccess = action.takeAction(docCopy, false);
+
+		if (pFilter != NULL)
+		{
+			delete pFilter;
+		}
+
+		return finalSuccess;
+	}
+
+	// At this point, pFilter cannot be NULL
+	bool hasDocs = pFilter->has_documents();
+#ifdef DEBUG
+	cout << "FilterUtils::filterDocument: has documents " << hasDocs << endl;
+#endif
+	while (hasDocs == true)
+	{
+		string actualType(originalType);
+		bool isNested = false;
+		bool emptyTitle = false;
+
+		if ((positionedFilter == false) &&
+			(pFilter->next_document() == false))
+		{
+#ifdef DEBUG
+			cout << "FilterUtils::filterDocument: no more documents in " << doc.getLocation(true) << endl;
+#endif
+			break;
+		}
+
+		string originalTitle(doc.getTitle());
+		Document filteredDoc(originalTitle, doc.getLocation(), "text/plain", doc.getLanguage());
+
+		filteredDoc.setInternalPath(doc.getInternalPath());
+		filteredDoc.setTimestamp(doc.getTimestamp());
+		filteredDoc.setSize(doc.getSize());
+		docSuccess = false;
+
+		if (populateDocument(filteredDoc, pFilter) == false)
+		{
+			hasDocs = pFilter->has_documents();
+			continue;
+		}
+
+		// Is this a nested document ?
+		if (filteredDoc.getInternalPath().length() > doc.getInternalPath().length())
+		{
+			actualType = filteredDoc.getType();
+#ifdef DEBUG
+			cout << "FilterUtils::filterDocument: nested document of type " << actualType << endl;
+#endif
+			isNested = true;
+		}
+		else if (originalTitle.empty() == false)
+		{
+			// Preserve the top-level document's title
+			filteredDoc.setTitle(originalTitle);
+		}
+		else if (filteredDoc.getTitle().empty() == true)
+		{
+			emptyTitle = true;
+		}
+
+		// Pass it down to another filter ?
+		if (action.isReduced(filteredDoc) == true)
+		{
+			// Do we need to set a default title ?
+			if (emptyTitle == true)
+			{
+				Url urlObj(doc.getLocation());
+
+				// Default to the file name as title
+				filteredDoc.setTitle(urlObj.getFile());
+#ifdef DEBUG
+				cout << "FilterUtils::filterDocument: set default title " << urlObj.getFile() << endl;
+#endif
+			}
+
+			filteredDoc.setType(actualType);
+
+			// Take the appropriate action
+			docSuccess = action.takeAction(filteredDoc, isNested);
+		}
+		else
+		{
+			docSuccess = filterDocument(filteredDoc, actualType, action);
+		}
+
+		// Consider indexing anything a success
+		if (docSuccess == true)
+		{
+			finalSuccess = true;
+		}
+
+		if (positionedFilter == true)
+		{
+			break;
+		}
+
+		// Next
+		hasDocs = pFilter->has_documents();
+	}
+
+	delete pFilter;
+
+#ifdef DEBUG
+	cout << "FilterUtils::filterDocument: done with " << doc.getLocation(true) << " status " << finalSuccess << endl;
+#endif
+
+	return finalSuccess;
+}
+
+bool FilterUtils::reduceDocument(const Document &doc, ReducedAction &action)
+{
+	string originalType(doc.getType());
+
+	return filterDocument(doc, originalType, action);
+}
+
 string FilterUtils::stripMarkup(const string &text)
 {
 	if (text.empty() == true)
@@ -398,11 +624,10 @@ string FilterUtils::stripMarkup(const string &text)
 	if ((feedFilter(doc, pFilter) == true) &&
 		(pFilter->next_document() == true))
 	{
-		const map<string, string> &metaData = pFilter->get_meta_data();
-		map<string, string>::const_iterator contentIter = metaData.find("content");
-		if (contentIter != metaData.end())
+		const dstring &content = pFilter->get_content();
+		if (content.empty() == false)
 		{
-			strippedText = contentIter->second;
+			strippedText = string(content.c_str(), content.length());
 		}
 	}
 
