@@ -1,5 +1,5 @@
 /*
- *  Copyright 2008 Fabrice Colin
+ *  Copyright 2008-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -42,28 +42,19 @@ TextConverter::~TextConverter()
 {
 }
 
-string TextConverter::toUTF8(const string &text, const string &charset)
-{
-	unsigned int textLen = (unsigned int)text.length();
-
-	// Call overload
-	return toUTF8(text.c_str(), textLen, charset);
-}
-
-string TextConverter::toUTF8(const char *pText, unsigned int textLen, const string &charset)
+dstring TextConverter::toUTF8(const dstring &text, string &charset)
 {
 	string textCharset(StringManip::toLowerCase(charset));
 	char outputBuffer[8192];
-	char *pInput = const_cast<char *>(pText);
+	char *pInput = const_cast<char *>(text.c_str());
 
 	m_conversionErrors = 0;
 
-	if ((pText == NULL) ||
-		(textLen == 0) ||
+	if ((text.empty() == true) ||
 		(textCharset == "utf-8"))
 	{
 		// No conversion necessary
-		return string(pText, textLen);
+		return text;
 	}
 
 	if (textCharset.empty() == true)
@@ -71,72 +62,104 @@ string TextConverter::toUTF8(const char *pText, unsigned int textLen, const stri
 		if (m_utf8Locale == true)
 		{
 			// The current locale uses UTF-8
-			return string(pText, textLen);
+			return text;
 		}
 
 		textCharset = m_localeCharset;
 	}
 
-	IConv converter("UTF-8", textCharset);
-	string outputText;
-	gsize inputSize = (gsize)textLen;
+	dstring outputText;
+	gsize inputSize = (gsize)text.length();
 	bool invalidSequence = false;
 
-	while (inputSize > 0)
+	try
 	{
-		char *pOutput = outputBuffer;
-		gsize outputSize = 8192;
-
-		size_t conversions = converter.iconv(&pInput, &inputSize, &pOutput, &outputSize);
-		int errorCode = errno;
-		if (conversions == -1)
+		IConv converter("UTF-8", textCharset);
+ 
+		while (inputSize > 0)
 		{
-			if (errorCode == EILSEQ)
-			{
-				// Conversion was only partially successful
-				++m_conversionErrors;
-#ifdef DEBUG
-				cout << "TextConverter::toUTF8: invalid sequence at " << pInput - pText << endl;
-#endif
-				if (m_conversionErrors >= m_maxErrors)
-				{
-					// Give up
-					return string(pText, textLen);
-				}
-				converter.reset();
+			char *pOutput = outputBuffer;
+			gsize outputSize = 8192;
 
-				outputText.append(outputBuffer, 8192 - outputSize);
-				if (invalidSequence == false)
-				{
-					outputText += "?";
-					invalidSequence = true;
-				}
-
-				// Skip that
-				++pInput;
-				--inputSize;
-				continue;
-			}
-			else if (errorCode != E2BIG)
+			size_t conversions = converter.iconv(&pInput, &inputSize, &pOutput, &outputSize);
+			int errorCode = errno;
+			if (conversions == -1)
 			{
+				if (errorCode == EILSEQ)
+				{
+					// Conversion was only partially successful
+					++m_conversionErrors;
 #ifdef DEBUG
-				cout << "TextConverter::toUTF8: unknown error " << errorCode << endl;
+					cout << "TextConverter::toUTF8: invalid sequence" << endl;
 #endif
-				return string(pText, textLen);
+					if (m_conversionErrors >= m_maxErrors)
+					{
+						// Give up
+						return text;
+					}
+					converter.reset();
+
+					outputText.append(outputBuffer, 8192 - outputSize);
+					if (invalidSequence == false)
+					{
+						outputText += "?";
+						invalidSequence = true;
+					}
+
+					// Skip that
+					++pInput;
+					--inputSize;
+					continue;
+				}
+				else if (errorCode != E2BIG)
+				{
+#ifdef DEBUG
+					cout << "TextConverter::toUTF8: unknown error " << errorCode << endl;
+#endif
+					return text;
+				}
 			}
-		}
-		else
-		{
-			invalidSequence = false;
+			else
+			{
+				invalidSequence = false;
+			}
+
+			// Append what was successfully converted
+			outputText.append(outputBuffer, 8192 - outputSize);
 		}
 
-		// Append what was successfully converted
-		outputText.append(outputBuffer, 8192 - outputSize);
+#ifdef DEBUG
+		cout << "TextConverter::toUTF8: " << m_conversionErrors << " conversion errors" << endl;
+#endif
 	}
+	catch (Error &ce)
+	{
 #ifdef DEBUG
-	cout << "TextConverter::toUTF8: " << m_conversionErrors << " conversion errors" << endl;
+		cout << "TextConverter::toUTF8: " << ce.what() << endl;
 #endif
+		outputText.clear();
 
+		string::size_type pos = textCharset.find('_');
+		if (pos != string::npos)
+		{
+			string fixedCharset(StringManip::replaceSubString(textCharset, "_", "-"));
+
+#ifdef DEBUG
+			cout << "TextConverter::toUTF8: trying with charset " << fixedCharset << endl;
+#endif
+			textCharset = fixedCharset;
+			outputText = toUTF8(text, fixedCharset);
+		}
+	}
+	catch (...)
+	{
+#ifdef DEBUG
+		cout << "TextConverter::toUTF8: unknown exception" << endl;
+#endif
+		outputText.clear();
+	}
+
+	charset = textCharset;
 	return outputText;
 }
 
@@ -151,10 +174,16 @@ string TextConverter::fromUTF8(const string &text)
 	{
 		return locale_from_utf8(text);
 	}
-	catch (ConvertError &ce)
+	catch (Error &ce)
 	{
 #ifdef DEBUG
 		cout << "TextConverter::fromUTF8: " << ce.what() << endl;
+#endif
+	}
+	catch (...)
+	{
+#ifdef DEBUG
+		cout << "TextConverter::fromUTF8: unknown exception" << endl;
 #endif
 	}
  
