@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007 Fabrice Colin
+ *  Copyright 2007-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,12 +25,18 @@
 
 #ifndef DIJON_FILTER_EXPORT
 #if defined __GNUC__ && (__GNUC__ >= 4)
-  #define DIJON_FILTER_EXPORT __attribute__ ((visibility("default")))
+  #define DIJON_FILTER_EXPORT     __attribute__ ((visibility("default")))
+  #define DIJON_FILTER_INITIALIZE __attribute__((constructor))
+  #define DIJON_FILTER_SHUTDOWN   __attribute__((destructor))
 #else
   #define DIJON_FILTER_EXPORT
+  #define DIJON_FILTER_INITIALIZE
+  #define DIJON_FILTER_SHUTDOWN
 #endif
 #endif
 
+#include "Memory.h"
+
 namespace Dijon
 {
     class Filter;
@@ -75,14 +81,14 @@ namespace Dijon
 	/** What data a filter supports as input.
 	 * It can be either the whole document data, its file name, or its URI.
 	 */
-	typedef enum { DOCUMENT_DATA=0, DOCUMENT_STRING, DOCUMENT_FILE_NAME, DOCUMENT_URI } DataInput;
+	typedef enum { DOCUMENT_DATA = 0, DOCUMENT_STRING, DOCUMENT_FILE_NAME, DOCUMENT_URI } DataInput;
 
 	/** Input properties supported by the filter.
-	 * - DEFAULT_CHARSET is the charset preferred by the client application.
+	 * - PREFERRED_CHARSET is the charset preferred by the client application.
 	 * The filter will convert document's content to this charset if possible.
 	 * - OPERATING_MODE can be set to either view or index.
 	 */
-	typedef enum { DEFAULT_CHARSET=0, OPERATING_MODE } Properties;
+	typedef enum { PREFERRED_CHARSET = 0, OPERATING_MODE } Properties;
 
 
 	// Information.
@@ -96,10 +102,6 @@ namespace Dijon
 
 	// Initialization.
 
-	/** Sets the UTF-8 conversion funtion.
-	 */
-	bool set_utf8_converter(convert_to_utf8_func *func);
-
 	/** Sets a property, prior to calling set_document_XXX().
 	 * Returns false if the property is not supported.
 	 */
@@ -160,10 +162,9 @@ namespace Dijon
 
 	/** Returns a dictionary of metadata extracted from the current document.
 	 * Metadata fields may include one or more of the following :
-	 * content, title, ipath, mimetype, language, charset, author, creator,
+	 * title, ipath, mimetype, language, charset, author, creator,
 	 * publisher, modificationdate, creationdate, size
 	 * Special considerations apply :
-	 * - content may contain binary data, watch out !
 	 * - ipath is an internal path to the nested document that can be
 	 * later passed to skip_to_document(). It may be empty if the parent
 	 * document's type doesn't allow embedding, in which case the filter
@@ -175,13 +176,16 @@ namespace Dijon
 	 */
 	const std::map<std::string, std::string> &get_meta_data(void) const;
 
+	/// Returns content.
+	const dstring &get_content(void) const;
+
     protected:
 	/// The MIME type handled by the filter.
 	std::string m_mimeType;
-	/// The UTF8 conversion function.
-	convert_to_utf8_func *m_convertToUTF8Func;
 	/// Metadata dictionary.
 	std::map<std::string, std::string> m_metaData;
+	/// Content.
+	dstring m_content;
 	/// The name of the input file, if any.
 	std::string m_filePath;
 
