@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2008 Fabrice Colin
+ *  Copyright 2007-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,12 +16,17 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <strings.h>
 #include <sys/types.h>
-#include <sys/stat.h>
+#ifdef HAVE_MMAP
+#include <sys/mman.h>
+#endif
 #include <time.h>
+#include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <iostream>
@@ -33,6 +38,7 @@
 #include "GMimeMboxFilter.h"
 
 using std::cout;
+using std::cerr;
 using std::endl;
 using std::string;
 using std::map;
@@ -54,7 +60,8 @@ DIJON_FILTER_EXPORT bool check_filter_data_input(int data_input)
 {
 	Filter::DataInput input = (Filter::DataInput)data_input;
 
-	if (input == Filter::DOCUMENT_FILE_NAME)
+	if ((input == Filter::DOCUMENT_DATA) ||
+		(input == Filter::DOCUMENT_FILE_NAME))
 	{
 		return true;
 	}
@@ -66,11 +73,67 @@ DIJON_FILTER_EXPORT Filter *get_filter(const std::string &mime_type)
 {
 	return new GMimeMboxFilter(mime_type);
 }
+
+DIJON_FILTER_INITIALIZE void initialize_gmime(void)
+{
+	// Initialize gmime
+#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
+	g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
+#else
+	g_mime_init(GMIME_INIT_FLAG_UTF8);
 #endif
+#ifdef DEBUG
+	cout << "GMimeMboxFilter: initialized" << endl;
+#endif
+}
+
+DIJON_FILTER_SHUTDOWN void shutdown_gmime(void)
+{
+	// Shutdown gmime
+	g_mime_shutdown();
+}
+#endif
+
+static bool read_stream(GMimeStream *memStream, dstring &fileBuffer)
+{
+	char readBuffer[4096];
+	ssize_t totalSize = 0, bytesRead = 0;
+	bool gotOutput = true;
+
+	do
+	{
+		bytesRead = g_mime_stream_read(memStream, readBuffer, 4096);
+		if (bytesRead > 0)
+		{
+			fileBuffer.append(readBuffer, bytesRead);
+			totalSize += bytesRead;
+		}
+		else if (bytesRead == -1)
+		{
+			// An error occured
+			if (errno != EINTR)
+			{
+				gotOutput = false;
+				break;
+			}
+
+			// Try again
+			bytesRead = 1;
+		}
+	} while (bytesRead > 0);
+#ifdef DEBUG
+	cout << "GMimeMboxFilter::extractPart: read " << totalSize
+		<< "/" << fileBuffer.size() << " bytes" << endl;
+#endif
+
+	return gotOutput;
+}
 
 GMimeMboxFilter::GMimeMboxFilter(const string &mime_type) :
 	Filter(mime_type),
 	m_returnHeaders(false),
+	m_pData(NULL),
+	m_dataLength(0),
 	m_fd(-1),
 	m_pGMimeMboxStream(NULL),
 	m_pParser(NULL),
@@ -80,21 +143,17 @@ GMimeMboxFilter::GMimeMboxFilter(const string &mime_type) :
 	m_messageStart(0),
 	m_foundDocument(false)
 {
-	// Initialize gmime
-	g_mime_init(GMIME_INIT_FLAG_UTF8);
 }
 
 GMimeMboxFilter::~GMimeMboxFilter()
 {
 	finalize(true);
-
-	// Shutdown gmime
-	g_mime_shutdown();
 }
 
 bool GMimeMboxFilter::is_data_input_ok(DataInput input) const
 {
-	if (input == DOCUMENT_FILE_NAME)
+	if ((input == DOCUMENT_DATA) ||
+		(input == DOCUMENT_FILE_NAME))
 	{
 		return true;
 	}
@@ -104,7 +163,7 @@ bool GMimeMboxFilter::is_data_input_ok(DataInput input) const
 
 bool GMimeMboxFilter::set_property(Properties prop_name, const string &prop_value)
 {
-	if (prop_name == DEFAULT_CHARSET)
+	if (prop_name == PREFERRED_CHARSET)
 	{
 		m_defaultCharset = prop_value;
 
@@ -129,7 +188,25 @@ bool GMimeMboxFilter::set_property(Properties prop_name, const string &prop_valu
 
 bool GMimeMboxFilter::set_document_data(const char *data_ptr, unsigned int data_length)
 {
-	return false;
+	// Close/free whatever was opened/allocated on a previous call to set_document()
+	finalize(true);
+	m_partsCount = m_partNum = -1;
+	m_messageStart = 0;
+	m_messageDate.clear();
+	m_partCharset.clear();
+	m_foundDocument = false;
+
+	m_pData = data_ptr;
+	m_dataLength = data_length;
+
+	// Assume there are documents if initialization is successful
+	// but don't actually retrieve anything, until next or skip is called
+	if (initializeData() == true)
+	{
+		m_foundDocument = initialize();
+	}
+
+	return m_foundDocument;
 }
 
 bool GMimeMboxFilter::set_document_string(const string &data_str)
@@ -151,7 +228,10 @@ bool GMimeMboxFilter::set_document_file(const string &file_path, bool unlink_whe
 
 	// Assume there are documents if initialization is successful
 	// but don't actually retrieve anything, until next or skip is called
-	m_foundDocument = initialize();
+	if (initializeFile() == true)
+	{
+		m_foundDocument = initialize();
+	}
 
 	return m_foundDocument;
 }
@@ -194,7 +274,7 @@ bool GMimeMboxFilter::skip_to_document(const string &ipath)
 	}
 
 	// ipath's format is "o=offset&p=part_number"
-	if (sscanf(ipath.c_str(), "o=%u&p=%d", &m_messageStart, &m_partNum) != 2)
+	if (sscanf(ipath.c_str(), "o=%u&p=%d", (unsigned int*)&m_messageStart, &m_partNum) != 2)
 	{
 		return false;
 	}
@@ -205,10 +285,14 @@ bool GMimeMboxFilter::skip_to_document(const string &ipath)
 	m_partCharset.clear();
 	m_foundDocument = false;
 
-	if (initialize() == true)
+	if (((m_filePath.empty() == false) && (initializeFile() == true)) ||
+		(initializeData() == true))
 	{
-		// Extract the first message at the given offset
-		m_foundDocument = extractMessage("");
+		if (initialize() == true)
+		{
+			// Extract the first message at the given offset
+			m_foundDocument = extractMessage("");
+		}
 	}
 
 	return m_foundDocument;
@@ -219,64 +303,110 @@ string GMimeMboxFilter::get_error(void) const
 	return "";
 }
 
-bool GMimeMboxFilter::initialize(void)
+bool GMimeMboxFilter::initializeData(void)
 {
-#ifndef O_NOATIME
-	int openFlags = O_RDONLY;
-#else
-	int openFlags = O_RDONLY|O_NOATIME;
-#endif
-
-	// Open the mbox file
-	m_fd = open(m_filePath.c_str(), openFlags);
-#ifdef O_NOATIME
-	if ((m_fd < 0) &&
-		(errno == EPERM))
+	// Create a stream
+	m_pGMimeMboxStream = g_mime_stream_mem_new_with_buffer(m_pData, m_dataLength);
+	if (m_pGMimeMboxStream == NULL)
 	{
-		// Try again
-		m_fd = open(m_filePath.c_str(), O_RDONLY);
-	}
-#endif
-	if (m_fd < 0)
-	{
-#ifdef DEBUG
-		cout << "GMimeMboxFilter::initialize: couldn't open " << m_filePath << endl;
-#endif
 		return false;
 	}
 
-	// Create a stream
 	if (m_messageStart > 0)
 	{
-		struct stat fileStat;
+		ssize_t streamLength = g_mime_stream_length(m_pGMimeMboxStream);
 
-		if ((fstat(m_fd, &fileStat) == 0) &&
-			(!S_ISREG(fileStat.st_mode)))
-		{
-			// This is not a file !
-			return false;
-		}
-
-		if (m_messageStart > fileStat.st_size)
+#ifdef DEBUG
+		cout << "GMimeMboxFilter::initializeData: from offset " << m_messageStart
+			<< " to " << streamLength << endl;
+#endif
+		if (m_messageStart > (GMIME_OFFSET_TYPE)streamLength)
 		{
 			// This offset doesn't make sense !
 			m_messageStart = 0;
 		}
 
-		m_pGMimeMboxStream = g_mime_stream_fs_new_with_bounds(m_fd, m_messageStart, fileStat.st_size);
+		g_mime_stream_set_bounds(m_pGMimeMboxStream, m_messageStart, (GMIME_OFFSET_TYPE)streamLength);
+	}
+
+	return true;
+}
+
+bool GMimeMboxFilter::initializeFile(void)
+{
+	int openFlags = O_RDONLY;
+#ifdef O_CLOEXEC
+	openFlags |= O_CLOEXEC;
+#endif
+
+	// Open the mbox file
+#ifdef O_NOATIME
+	m_fd = open(m_filePath.c_str(), openFlags|O_NOATIME);
+#else
+	m_fd = open(m_filePath.c_str(), openFlags);
+#endif
+#ifdef O_NOATIME
+	if ((m_fd < 0) &&
+		(errno == EPERM))
+	{
+		// Try again
+		m_fd = open(m_filePath.c_str(), openFlags);
+	}
+#endif
+	if (m_fd < 0)
+	{
 #ifdef DEBUG
-		cout << "GMimeMboxFilter::initialize: stream starts at offset " << m_messageStart << endl;
+		cout << "GMimeMboxFilter::initializeFile: couldn't open " << m_filePath << endl;
+#endif
+		return false;
+	}
+#ifndef O_CLOEXEC
+	int fdFlags = fcntl(m_fd, F_GETFD);
+	fcntl(m_fd, F_SETFD, fdFlags|FD_CLOEXEC);
+#endif
+
+	// Create a stream
+	if (m_messageStart > 0)
+	{
+		ssize_t streamLength = g_mime_stream_length(m_pGMimeMboxStream);
+
+		if (m_messageStart > (GMIME_OFFSET_TYPE)streamLength)
+		{
+			// This offset doesn't make sense !
+			m_messageStart = 0;
+		}
+
+#ifdef HAVE_MMAP
+		m_pGMimeMboxStream = g_mime_stream_mmap_new_with_bounds(m_fd, PROT_READ, MAP_PRIVATE, m_messageStart, (GMIME_OFFSET_TYPE)streamLength);
+#else
+		m_pGMimeMboxStream = g_mime_stream_fs_new_with_bounds(m_fd, m_messageStart, (GMIME_OFFSET_TYPE)streamLength);
+#endif
+#ifdef DEBUG
+		cout << "GMimeMboxFilter::initializeFile: stream starts at offset " << m_messageStart << endl;
 #endif
 	}
 	else
 	{
+#ifdef HAVE_MMAP
+		m_pGMimeMboxStream = g_mime_stream_mmap_new(m_fd, PROT_READ, MAP_PRIVATE);
+#else
 		m_pGMimeMboxStream = g_mime_stream_fs_new(m_fd);
+#endif
+	}
+
+	return true;
+}
+
+bool GMimeMboxFilter::initialize(void)
+{
+	if (m_pGMimeMboxStream == NULL)
+	{
+		return false;
 	}
 
 	// And a parser
 	m_pParser = g_mime_parser_new();
-	if ((m_pGMimeMboxStream != NULL) &&
-		(m_pParser != NULL))
+	if (m_pParser != NULL)
 	{
 		g_mime_parser_init_with_stream(m_pParser, m_pGMimeMboxStream);
 		g_mime_parser_set_respect_content_length(m_pParser, TRUE);
@@ -296,39 +426,115 @@ void GMimeMboxFilter::finalize(bool fullReset)
 {
 	if (m_pMimeMessage != NULL)
 	{
+#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
+		if (G_IS_OBJECT(m_pMimeMessage))
+		{
+			g_object_unref(m_pMimeMessage);
+		}
+#else
 		g_mime_object_unref(GMIME_OBJECT(m_pMimeMessage));
+#endif
 		m_pMimeMessage = NULL;
 	}
 	if (m_pParser != NULL)
 	{
 		// FIXME: does the parser close the stream ?
-		g_object_unref(G_OBJECT(m_pParser));
+		if (G_IS_OBJECT(m_pParser))
+		{
+			g_object_unref(m_pParser);
+		}
 		m_pParser = NULL;
 	}
-	else if (m_pGMimeMboxStream != NULL)
+	if (m_pGMimeMboxStream != NULL)
 	{
-		g_object_unref(G_OBJECT(m_pGMimeMboxStream));
+		if (G_IS_OBJECT(m_pGMimeMboxStream))
+		{
+			g_object_unref(m_pGMimeMboxStream);
+		}
 		m_pGMimeMboxStream = NULL;
 	}
+
+	// initializeFile() will always reopen the file
 	if (m_fd >= 0)
 	{
 		close(m_fd);
 		m_fd = -1;
 	}
-
 	if (fullReset == true)
 	{
+		// ...but those data fields will only be reinit'ed on a full reset
+		m_pData = NULL;
+		m_dataLength = 0;
+
 		rewind();
 	}
 }
 
-char *GMimeMboxFilter::extractPart(GMimeObject *part, string &contentType, ssize_t &partLen)
+bool GMimeMboxFilter::nextPart(const string &subject)
 {
-	char *pBuffer = NULL;
+	if (m_pMimeMessage != NULL)
+	{
+		// Get the top-level MIME part in the message
+		GMimeObject *pMimePart = g_mime_message_get_mime_part(m_pMimeMessage);
+		if (pMimePart != NULL)
+		{
+			string partTitle(subject), partContentType;
 
+			// Extract the part's text
+			m_content.clear();
+			if (extractPart(pMimePart, partTitle, partContentType, m_content) == true)
+			{
+				char posStr[128];
+
+				// New document
+				m_metaData.clear();
+				m_metaData["title"] = partTitle;
+				m_metaData["mimetype"] = partContentType;
+				m_metaData["date"] = m_messageDate;
+				m_metaData["charset"] = m_partCharset;
+				snprintf(posStr, 128, "%u", m_content.length());
+				m_metaData["size"] = posStr;
+				// FIXME: use the same scheme as Mozilla
+				snprintf(posStr, 128, "o=%u&p=%d", m_messageStart, max(m_partNum - 1, 0));
+				m_metaData["ipath"] = posStr;
+#ifdef DEBUG
+				cout << "GMimeMboxFilter::extractMessage: message location is " << posStr << endl; 
+#endif
+
+#ifndef GMIME_ENABLE_RFC2047_WORKAROUNDS
+				g_mime_object_unref(pMimePart);
+#endif
+
+				return true;
+			}
+
+#ifndef GMIME_ENABLE_RFC2047_WORKAROUNDS
+			g_mime_object_unref(pMimePart);
+#endif
+		}
+
+#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
+		if (G_IS_OBJECT(m_pMimeMessage))
+		{
+			g_object_unref(m_pMimeMessage);
+		}
+#else
+		g_mime_object_unref(GMIME_OBJECT(m_pMimeMessage));
+#endif
+		m_pMimeMessage = NULL;
+	}
+
+	// If we get there, no suitable parts were found
+	m_partsCount = m_partNum = -1;
+
+	return false;
+}
+
+bool GMimeMboxFilter::extractPart(GMimeObject *part, string &subject, string &contentType, dstring &partBuffer)
+{
 	if (part == NULL)
 	{
-		return NULL;
+		return false;
 	}
 
 	// Message parts may be nested
@@ -339,13 +545,19 @@ char *GMimeMboxFilter::extractPart(GMimeObject *part, string &contentType, ssize
 #endif
 		GMimeMessage *partMessage = g_mime_message_part_get_message(GMIME_MESSAGE_PART(part));
 		part = g_mime_message_get_mime_part(partMessage);
+#ifndef GMIME_ENABLE_RFC2047_WORKAROUNDS
 		g_mime_object_unref(GMIME_OBJECT(partMessage));
+#endif
 	}
 
 	// Is this a multipart ?
 	if (GMIME_IS_MULTIPART(part))
 	{
+#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
+		m_partsCount = g_mime_multipart_get_count(GMIME_MULTIPART(part));
+#else
 		m_partsCount = g_mime_multipart_get_number(GMIME_MULTIPART(part));
+#endif
 #ifdef DEBUG
 		cout << "GMimeMboxFilter::extractPart: message has " << m_partsCount << " parts" << endl;
 #endif
@@ -361,12 +573,15 @@ char *GMimeMboxFilter::extractPart(GMimeObject *part, string &contentType, ssize
 				continue;
 			}
 
-			char *pPart = extractPart(multiMimePart, contentType, partLen);
+			bool gotPart = extractPart(multiMimePart, subject, contentType, partBuffer);
+#ifndef GMIME_ENABLE_RFC2047_WORKAROUNDS
 			g_mime_object_unref(multiMimePart);
-			if (pPart != NULL)
+#endif
+
+			if (gotPart == true)
 			{
 				m_partNum = ++partNum;
-				return pPart;
+				return true;
 			}
 		}
 
@@ -379,13 +594,16 @@ char *GMimeMboxFilter::extractPart(GMimeObject *part, string &contentType, ssize
 #ifdef DEBUG
 		cout << "GMimeMboxFilter::extractPart: not a part" << endl;
 #endif
-		return NULL;
+		return false;
 	}
 	GMimePart *mimePart = GMIME_PART(part);
-	GMimeFilter *charsetFilter = NULL;
 
 	// Check the content type
-	const GMimeContentType *mimeType = g_mime_part_get_content_type(mimePart);
+#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
+	GMimeContentType *mimeType = g_mime_object_get_content_type(GMIME_OBJECT(mimePart));
+#else
+	const GMimeContentType *mimeType = g_mime_object_get_content_type(GMIME_OBJECT(mimePart));
+#endif
 	// Set this for caller
 	char *partType = g_mime_content_type_to_string(mimeType);
 	if (partType != NULL)
@@ -397,11 +615,27 @@ char *GMimeMboxFilter::extractPart(GMimeObject *part, string &contentType, ssize
 		g_free(partType);
 	}
 
+#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
+	GMimeContentEncoding encodingType = g_mime_part_get_content_encoding(mimePart);
+#else
 	GMimePartEncodingType encodingType = g_mime_part_get_encoding(mimePart);
+#endif
 #ifdef DEBUG
 	cout << "GMimeMboxFilter::extractPart: encoding is " << encodingType << endl;
 #endif
+#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
+	g_mime_part_set_content_encoding(mimePart, GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE);
+#else
 	g_mime_part_set_encoding(mimePart, GMIME_PART_ENCODING_QUOTEDPRINTABLE);
+#endif
+	const char *fileName = g_mime_part_get_filename(mimePart);
+	if (fileName != NULL)
+	{
+#ifdef DEBUG
+		cout << "GMimeMboxFilter::extractPart: file name is " << fileName << endl;
+#endif
+		subject = fileName;
+	}
 
 	// Create a in-memory output stream
 	GMimeStream *memStream = g_mime_stream_mem_new();
@@ -414,7 +648,7 @@ char *GMimeMboxFilter::extractPart(GMimeObject *part, string &contentType, ssize
 		// Install a charset filter
 		if (strncasecmp(charset, "UTF-8", 5) != 0)
 		{
-			charsetFilter = g_mime_filter_charset_new(charset, "UTF-8");
+			GMimeFilter *charsetFilter = g_mime_filter_charset_new(charset, "UTF-8");
 			if (charsetFilter != NULL)
 			{
 #ifdef DEBUG
@@ -435,31 +669,45 @@ char *GMimeMboxFilter::extractPart(GMimeObject *part, string &contentType, ssize
 #ifdef DEBUG
 		cout << "GMimeMboxFilter::extractPart: wrote " << writeLen << " bytes" << endl;
 #endif
-		g_object_unref(dataWrapper);
+		if (G_IS_OBJECT(dataWrapper))
+		{
+			g_object_unref(dataWrapper);
+		}
 	}
 	g_mime_stream_flush(memStream);
-	partLen = g_mime_stream_length(memStream);
+	ssize_t partLen = g_mime_stream_length(memStream);
 #ifdef DEBUG
 	cout << "GMimeMboxFilter::extractPart: part is " << partLen << " bytes long" << endl;
 #endif
 
-	pBuffer = (char*)malloc(partLen + 1);
-	pBuffer[partLen] = '\0';
-	g_mime_stream_reset(memStream);
-	ssize_t readLen = g_mime_stream_read(memStream, pBuffer, partLen);
-#ifdef DEBUG
-	cout << "GMimeMboxFilter::extractPart: read " << readLen << " bytes" << endl;
-#endif
-	g_mime_stream_unref(memStream);
+	if ((m_returnHeaders == true) &&
+		(contentType.length() >= 10) &&
+		(strncasecmp(contentType.c_str(), "text/plain", 10) == 0))
+	{
+		char *pHeaders = g_mime_object_get_headers(GMIME_OBJECT(m_pMimeMessage));
 
-	return pBuffer;
+		if (pHeaders != NULL)
+		{
+			partBuffer = pHeaders;
+			partBuffer += "\n";
+			free(pHeaders);
+		}
+	}
+
+	g_mime_stream_reset(memStream);
+	partBuffer.reserve(partLen);
+	read_stream(memStream, partBuffer);
+	if (G_IS_OBJECT(memStream))
+	{
+		g_object_unref(memStream);
+	}
+
+	return true;
 }
 
 bool GMimeMboxFilter::extractMessage(const string &subject)
 {
-	string msgSubject(subject), contentType;
-	char *pPart = NULL;
-	ssize_t partLength = 0;
+	string msgSubject(subject);
 
 	while (g_mime_stream_eos(m_pGMimeMboxStream) == FALSE)
 	{
@@ -469,31 +717,48 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 			// No, it doesn't
 			if (m_pMimeMessage != NULL)
 			{
+#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
+				if (G_IS_OBJECT(m_pMimeMessage))
+				{
+					g_object_unref(m_pMimeMessage);
+				}
+#else
 				g_mime_object_unref(GMIME_OBJECT(m_pMimeMessage));
+#endif
 				m_pMimeMessage = NULL;
 			}
 
 			// Get the next message
 			m_pMimeMessage = g_mime_parser_construct_message(m_pParser);
+			if (m_pMimeMessage == NULL)
+			{
+				cerr << "Couldn't construct new MIME message" << endl;
+				break;
+			}
 
 			m_messageStart = g_mime_parser_get_from_offset(m_pParser);
+#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
+			gint64 messageEnd = g_mime_parser_tell(m_pParser);
+#else
 			off_t messageEnd = g_mime_parser_tell(m_pParser);
+#endif
 #ifdef DEBUG
 			cout << "GMimeMboxFilter::extractMessage: message between offsets " << m_messageStart
 				<< " and " << messageEnd << endl;
 #endif
 			if (messageEnd > m_messageStart)
 			{
-				// FIXME: this only applies to Mozilla
-				const char *pMozStatus = g_mime_message_get_header(m_pMimeMessage, "X-Mozilla-Status");
+				// This only applies to Mozilla
+				const char *pMozStatus = g_mime_object_get_header(GMIME_OBJECT(m_pMimeMessage), "X-Mozilla-Status");
 				if (pMozStatus != NULL)
 				{
-					long int mozStatus = strtol(pMozStatus, NULL, 16);
+					long int mozFlags = strtol(pMozStatus, NULL, 16);
+
 					// Watch out for Mozilla specific flags :
 					// MSG_FLAG_EXPUNGED, MSG_FLAG_EXPIRED
 					// They are defined in mailnews/MailNewsTypes.h and msgbase/nsMsgMessageFlags.h
-					if ((mozStatus & 0x0008) ||
-						(mozStatus & 0x0040))
+					if ((mozFlags & 0x0008) ||
+						(mozFlags & 0x0040))
 					{
 #ifdef DEBUG
 						cout << "GMimeMboxFilter::extractMessage: flagged by Mozilla" << endl;
@@ -501,9 +766,32 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 						continue;
 					}
 				}
+				// This only applies to Evolution
+				const char *pEvoStatus = g_mime_object_get_header(GMIME_OBJECT(m_pMimeMessage), "X-Evolution");
+				if (pEvoStatus != NULL)
+				{
+					string evoStatus(pEvoStatus);
+					string::size_type flagsPos = evoStatus.find('-');
+
+					if (flagsPos != string::npos)
+					{
+						long int evoFlags = strtol(evoStatus.substr(flagsPos + 1).c_str(), NULL, 16);
+
+						// Watch out for Evolution specific flags :
+						// CAMEL_MESSAGE_DELETED
+						// It's defined in camel/camel-folder-summary.h
+						if (evoFlags & 0x0002)
+						{
+#ifdef DEBUG
+							cout << "GMimeMboxFilter::extractMessage: flagged by Evolution" << endl;
+#endif
+							continue;
+						}
+					}
+				}
 
 				// How old is this message ?
-				const char *pDate = g_mime_message_get_header(m_pMimeMessage, "Date");
+				const char *pDate = g_mime_object_get_header(GMIME_OBJECT(m_pMimeMessage), "Date");
 				if (pDate != NULL)
 				{
 					m_messageDate = pDate;
@@ -511,17 +799,24 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 				else
 				{
 					time_t timeNow = time(NULL);
-					struct tm timeTm;
+					struct tm *pTimeTm = new struct tm;
 
-					if (localtime_r(&timeNow, &timeTm) != NULL)
+#ifdef HAVE_LOCALTIME_R
+					if (localtime_r(&timeNow, pTimeTm) != NULL)
+#else
+					pTimeTm = localtime(&timeNow);
+					if (pTimeTm != NULL)
+#endif
 					{
 						char timeStr[64];
 
-						if (strftime(timeStr, 64, "%a, %d %b %Y %H:%M:%S %Z", &timeTm) > 0)
+						if (strftime(timeStr, 64, "%a, %d %b %Y %H:%M:%S %Z", pTimeTm) > 0)
 						{
 							m_messageDate = timeStr;
 						}
 					}
+
+					delete pTimeTm;
 				}
 #ifdef DEBUG
 				cout << "GMimeMboxFilter::extractMessage: message date is " << m_messageDate << endl;
@@ -539,70 +834,13 @@ bool GMimeMboxFilter::extractMessage(const string &subject)
 		cout << "GMimeMboxFilter::extractMessage: message subject is " << msgSubject << endl;
 #endif
 
-		if (m_pMimeMessage != NULL)
-		{
-			// Get the top-level MIME part in the message
-			GMimeObject *pMimePart = g_mime_message_get_mime_part(m_pMimeMessage);
-			if (pMimePart != NULL)
-			{
-				// Extract the part's text
-				pPart = extractPart(pMimePart, contentType, partLength);
-				if (pPart != NULL)
-				{
-					string content;
-					string location("mailbox://");
-					char posStr[128];
+		return nextPart(msgSubject);
+	}
 
-					if ((m_returnHeaders == true) &&
-						(contentType.length() >= 10) &&
-						(strncasecmp(contentType.c_str(), "text/plain", 10) == 0))
-					{
-						char *pHeaders = g_mime_message_get_headers(m_pMimeMessage);
-
-						if (pHeaders != NULL)
-						{
-							content = pHeaders;
-							content += "\n";
-							free(pHeaders);
-						}
-					}
-					content += string(pPart, (unsigned int)partLength);
-
-					// FIXME: use the same scheme as Mozilla
-					location += m_filePath;
-
-					// New document
-					m_metaData.clear();
-					m_metaData["title"] = msgSubject;
-					m_metaData["uri"] = location;
-					m_metaData["mimetype"] = contentType;
-					m_metaData["content"] = content;
-					m_metaData["date"] = m_messageDate;
-					m_metaData["charset"] = m_partCharset;
-					snprintf(posStr, 128, "%u", partLength);
-					m_metaData["size"] = posStr;
-					snprintf(posStr, 128, "o=%u&p=%d", m_messageStart, max(m_partNum - 1, 0));
-					m_metaData["ipath"] = posStr;
-#ifdef DEBUG
-					cout << "GMimeMboxFilter::extractMessage: message location is " << location
-						<< "?" << posStr << endl;
-#endif
-
-					free(pPart);
-					g_mime_object_unref(pMimePart);
-
-					return true;
-				}
-
-				g_mime_object_unref(pMimePart);
-			}
-
-			g_mime_object_unref(GMIME_OBJECT(m_pMimeMessage));
-			m_pMimeMessage = NULL;
-		}
-
-		// If we get there, no suitable parts were found
-		m_partsCount = m_partNum = -1;
+	// The last message may have parts left
+	if (m_partsCount != -1)
+	{
+		return nextPart(msgSubject);
 	}
 
 	return false;
