@@ -53,6 +53,7 @@ using namespace std;
 static ofstream g_outputFile;
 static streambuf *g_coutBuf = NULL;
 static streambuf *g_cerrBuf = NULL;
+static streambuf *g_clogBuf = NULL;
 static struct option g_longOptions[] = {
 	{"help", 0, 0, 'h'},
 	{"preferences", 0, 0, 'p'},
@@ -76,6 +77,10 @@ static void closeAll(void)
 	{
 		cerr.rdbuf(g_cerrBuf);
 	}
+	if (g_clogBuf != NULL)
+	{
+		clog.rdbuf(g_cerrBuf);
+	}
 	g_outputFile.close();
 
 	DownloaderInterface::shutdown();
@@ -89,6 +94,7 @@ static void quitAll(int sigNum)
 	Gtk::Main::quit();
 }
 
+#ifdef HAVE_DBUS
 static DBusHandlerResult filterHandler(DBusConnection *pConnection, DBusMessage *pMessage, void *pData)
 {
 	// Are we about to be disconnected ?
@@ -141,6 +147,7 @@ static DBusHandlerResult filterHandler(DBusConnection *pConnection, DBusMessage
 
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
+#endif
 
 int main(int argc, char **argv)
 {
@@ -278,8 +285,10 @@ int main(int argc, char **argv)
 		g_outputFile.open(logFileName.c_str());
 		g_coutBuf = cout.rdbuf();
 		g_cerrBuf = cerr.rdbuf();
+		g_clogBuf = clog.rdbuf();
 		cout.rdbuf(g_outputFile.rdbuf());
 		cerr.rdbuf(g_outputFile.rdbuf());
+		clog.rdbuf(g_outputFile.rdbuf());
 	}
 
 	// Initialize utility classes
@@ -381,6 +390,7 @@ int main(int argc, char **argv)
 
 	if (prefsMode == false)
 	{
+#ifdef HAVE_DBUS
 		GError *pError = NULL;
 		DBusGConnection *pBus = dbus_g_bus_get(DBUS_BUS_SESSION, &pError);
 		DBusConnection *pConnection = NULL;
@@ -422,6 +432,7 @@ int main(int argc, char **argv)
 
 			dbus_error_free(&error);
 		}
+#endif
 
 		IndexInterface *pIndex = settings.getIndex(settings.m_docsIndexLocation);
 		if (pIndex != NULL)
