@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2008 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,11 +40,13 @@ class PINOT_EXPORT DocumentInfo
 
 		bool operator<(const DocumentInfo& other) const;
 
+		typedef enum { SERIAL_ALL = 0, SERIAL_FIELDS, SERIAL_LABELS } SerialExtent;
+
 		/// Serializes the document.
-		std::string serialize(void) const;
+		std::string serialize(SerialExtent extent = SERIAL_ALL) const;
 
 		/// Deserializes the document.
-		void deserialize(const std::string &info);
+		void deserialize(const std::string &info, SerialExtent extent = SERIAL_ALL);
 
 		/// Sets the title of the document.
 		virtual void setTitle(const std::string &title);
@@ -56,7 +58,13 @@ class PINOT_EXPORT DocumentInfo
 		virtual void setLocation(const std::string &location);
 
 		/// Returns the original location of the document.
-		virtual std::string getLocation(void) const;
+		virtual std::string getLocation(bool withIPath = false) const;
+
+		/// Sets the internal path to the document.
+		void setInternalPath(const std::string &ipath);
+
+		/// Returns the internal path to the document.
+		std::string getInternalPath(void) const;
 
 		/// Sets the type of the document.
 		virtual void setType(const std::string &type);
